@@ -55,36 +55,13 @@ static void map_statusbar(map_t *map, map_item_t *map_item) {
 
   gboolean collision = FALSE;
   tag_t *tags = tag;
-
-  if(id == ID_ILLEGAL) 
-    str = g_strdup_printf(_("Unknown item"));
-  else {
-    str = g_strdup_printf("%s #" ITEM_ID_FORMAT, item_str, id);
-
-    /* add some tags ... */
-    /*
-     *  XXX Should we just try to present only the name or the ref (or the
-     *  alt_name, old_name, whatever) here?  Hurling a load of tags in the
-     *  user's face in some unpredictable, uninformative order isn't very
-     *  friendly.
-     *
-     *  Actually, a tag_short_desc() function would be useful in dialogs
-     *  nd user messages too.
-     */
-    while(tag) {
-      if(!collision && info_tag_key_collision(tags, tag))
-	collision = TRUE;
-
-      /* we don't have much space, so ignore created_by tag */
-      if(!osm_is_creator_tag(tag)) {
-	char *old = str;
-	str = g_strdup_printf("%s, %s=%s", old, tag->key, tag->value);
-	g_free(old);
-      }
-      tag = tag->next;
-    }
+  while(tag) {
+    if(!collision && info_tag_key_collision(tags, tag))
+      collision = TRUE;
+    tag = tag->next;
   }
-  
+
+  str = osm_object_get_speaking_name(&map_item->object);
   statusbar_set(map->appdata, str, collision);
   g_free(str);
 }
@@ -685,6 +662,11 @@ static void map_item_init(style_t *style, map_item_t *map_item) {
 void map_item_redraw(appdata_t *appdata, map_item_t *map_item) {
   map_item_t item = *map_item;
 
+  /* a relation cannot be redraws as it doesn't have a visual */
+  /* representation */
+  if(map_item->object.type == RELATION)
+    return;
+
   /* check if the item to be redrawn is the selected one */
   gboolean is_selected = FALSE;
   if(map_item->object.ptr == appdata->map->selected.object.ptr) {
@@ -1790,8 +1772,7 @@ GtkWidget *map_new(appdata_t *appdata) {
   map->appdata = appdata;
   map->action.type = MAP_ACTION_IDLE;
 
-  map->canvas = canvas_new(map->style->background.color);
-  canvas_set_antialias(map->canvas, !appdata->settings->no_antialias);
+  map->canvas = canvas_new();
 
   GtkWidget *canvas_widget = canvas_get_widget(map->canvas);
 
@@ -1820,6 +1801,9 @@ GtkWidget *map_new(appdata_t *appdata) {
 void map_init(appdata_t *appdata) {
   map_t *map = appdata->map;
 
+  /* update canvas background color */
+  canvas_set_background(map->canvas, map->style->background.color);
+
   /* set initial zoom */
   map_set_zoom(map, map->state->zoom, FALSE);
   josm_elemstyles_colorize_world(map->style, appdata->osm);
@@ -1859,9 +1843,6 @@ void map_clear(appdata_t *appdata, gint group_mask) {
 void map_paint(appdata_t *appdata) {
   map_t *map = appdata->map;
 
-  /* user may have changed antialias settings */
-  canvas_set_antialias(map->canvas, !appdata->settings->no_antialias);
-
   josm_elemstyles_colorize_world(map->style, appdata->osm);
   map_draw(map, appdata->osm);
 }
