@@ -446,9 +446,10 @@ static GtkWidget *relation_item_list_widget(relitem_context_t *context) {
   
   g_object_unref(context->store);
 
-  list_set_static_buttons(context->list, G_CALLBACK(on_relation_item_add),
-	G_CALLBACK(on_relation_item_edit),G_CALLBACK(on_relation_item_remove), 
-        context);
+  list_set_static_buttons(context->list, FALSE, 
+    G_CALLBACK(on_relation_item_add),
+    G_CALLBACK(on_relation_item_edit),G_CALLBACK(on_relation_item_remove), 
+			  context);
 
   relation_item_list_selected(context, FALSE);
 
@@ -903,7 +904,7 @@ static GtkWidget *relation_list_widget(relation_context_t *context) {
   
   g_object_unref(context->store);
 
-  list_set_static_buttons(context->list, G_CALLBACK(on_relation_add),
+  list_set_static_buttons(context->list, TRUE, G_CALLBACK(on_relation_add),
      G_CALLBACK(on_relation_edit), G_CALLBACK(on_relation_remove), context);
 
   list_set_user_buttons(context->list, 
